//Team: 	Revolver 




/**
 * @file 
 * 	This file is the implementation of all the member functions of the 
 * 	cFunc class.
 * @brief
 * 	class Cfunc function definitions
*/


//Include header file
#include "cFunc.h"

//************************* Function Definitions ************************

/**
 * This function is the constructor for the Cfunc class, it initializes
 * Cfunc_HomeDir
*/
Cfunc::Cfunc()
{
	read = O_RDONLY;
	write = O_WRONLY;
	readWrite = O_RDWR;
	create = O_CREAT;
	append = O_APPEND;
	homeDir = getenv("HOME");
} //end of Cfunc construtor


/**
 * This function returns the fHomeDir member of the Cfunc class
 * @return string
*/
string Cfunc::getHome()
{
	return homeDir;
} //end of getHome

/**
 * This function takes in a string, converts it to a char * and
 * tries to open it, using mode given, returning true if all went
 * well and false if there was an error
 * @param inFileName a string
 * @param inFlags an int
 * @return bool
*/
bool Cfunc::canOpenFile(string inFileName, int inFlags)
{

	/** @pre TRUE */
	/** @post Returns true if file inFileName can be opened with
				 inFlags flags */

	//convert to a char *
	const char *fileName = inFileName.c_str();

	//store the return value
	int fid = open(fileName, inFlags, 00600);
	close(fid);
	
	//return true if fid != -1, false otherwise
	return (fid != -1);
	
}//end of canOpenFile function


/**
 * This function takes in a string, converts it to a char *, then 
 * renames it appending a ~, and creates an empty file with original name
 * @param inFileNam a string
 * @return bool
*/
bool Cfunc::createBackup(string inFileName)
{

	/** @pre inFileName is a valid filename */
	/** @post Returns true if backup file was sucessfully created,
				 false otherwise */

	//first translate oldFileName into a char *
	const char *oldFileName = inFileName.c_str();

	//then translate newFileName into a char *	
	string newFileNameStr = inFileName + "~";
	const char *newFileName = newFileNameStr.c_str();

	//rename the existing file
	int retVal = 0;
	retVal = rename(oldFileName, newFileName);
//	if(retVal != 0)
//		return false;	

	//create an empty file with oldFileName
	int fid = open(oldFileName, O_WRONLY | O_CREAT, 00600);
//	if(fid == -1)
//		return false;

	//otherwise all is good, close file and return true
	close(fid);
	return true;

}//end of createBackup function


/**
 * This function takes in two strings representing file names
 * it then opens them both, and compares their sizes and returns
 * whether they are the same size or not
 * @param inRefFile a string
 * @param inTestFile a string
 * @return bool
*/
bool Cfunc::areFilesSameSize(string inRefFile, string inTestFile)
{

	/** @pre inRefFIle and inTestFile are valid filenames */
	/** @post returns true if files have same size, false otherwise */

	//first convert filename strings to char *'s
	const char *refFile = inRefFile.c_str();
	const char *testFile = inTestFile.c_str();

	//then open them both up
	int fidRef = open(refFile, O_RDONLY);
	int fidTest = open(testFile, O_RDONLY);

	//now store their sizes
	off_t offset = 0;
	off_t sizeRef = lseek(fidRef, offset, SEEK_END);
	off_t sizeTest = lseek(fidTest, offset, SEEK_END);

	//close the two files
	close(fidRef);
	close(fidTest);

	//return if they are the same or not
	return (sizeRef == sizeTest);

}//end of areFilesSameSize


/**
 * This function takes in nothing, and creates an empty
 * temp file named 'fd.temp', it reutnrs true on sucess
 * false otherwise
 * @return bool
*/
bool Cfunc::createTempFile()
{

	/** @pre TRUE */
	/** @post Returns true if file "fd.temp" was created sucesfully,
				 false otherwise */

	int fid = open("fd.temp", O_CREAT | O_RDWR, S_IRWXU);

//	if(fid == -1)
//		return false;

	close(fid);
	return true;

}//end of createTempFile function


/**
 * This function deletes the temporary file created by
 * createTempFile
*/
void Cfunc::deleteTempFile()
{

	/** @pre TRUE */
	/** @post There will be no more file named "fd.temp" */
	
	system("rm -f fd.temp");

}//end of deleteTempFile function

//***********************************************************************
