//Team: 	Revolver 




/** 
 * @file
 * 	This file is the main implementation for the fd program
 * 	or at least for the first part of it.
 * @brief
 * 	main implementation of fd
*/
		
//************************* Includes and namespace ***********************
#include "/usr/local/pkgs/boost-1.30.2/boost/filesystem/operations.hpp"
#include "/usr/local/pkgs/boost-1.30.2/boost/filesystem/fstream.hpp"
#include <iostream>
#include <vector>
#include <string>
#include "fd.h"
#include "cFunc.h"

/**
 * @brief boost::filesystem namespace
 * @namespace fs
 * This namespace is the boost::filesystem namespace used for objects
 * and functions taken from the boost filesystem library
*/
namespace fs = boost::filesystem;
using namespace std;
//************************************************************************


//************************** Function Prototypes *************************

/// stores paths from user input
vector<fs::path> getPathsFromUser();


//******************************** Main **********************************
int main(int argc, char* argv[])
{
   
	//Big try-catch to catch pre/post condition failures
	try
	{

		//FDobject
		FD fdObj;	

   	fdObj.parseCommandLine(argc, argv);

		vector<fs::path> searchPaths;

		//if there are no arguments, then use old functionality
		if( !fdObj.areCmdLinePaths() )
			searchPaths = getPathsFromUser();	
		
		//otherwise get paths from command line arguments
		else
			searchPaths = fdObj.getPathsFromArgs();

		//if there are no files to search, tell user he goofed and exit
		if(searchPaths.size() == 0)
		{
			cout << "No valid paths entered! Paths needed to find duplicates!\n";
			exit(0);
		}

		//before the search, get rid of redundant paths
		removeRedundantPaths(searchPaths);

		//search all files from all searchPaths
		fdObj.startSearch(searchPaths);

		//print duplicate sub-trees
		fdObj.printDupDirs();

		//Print out file duplicates
		fdObj.printDups();

		}

		//catch preConditionFailed
		catch(PreConditionFailed pre)
		{
			cerr << "Pre-condition failed for: " << pre.method << endl;
			return 1;
		}
		
		//catch postConditionFailed
		catch(PostConditionFailed post)
		{
			cerr << "Post-condition failed for: " << post.method << endl;
			return 1; 
		}

		return 0;

} // end of main


//************************************************************************


//************************* Function Definitions *************************

/**
 * This function reads in paths from standard input 
 * until it encounters EOF
 * @return vector<fs::path>
*/
vector<fs::path> getPathsFromUser()
{

	/** @pre TRUE */
	/** @post All paths entered by user are returned */

	const int kStringSize = 256;
	
	//vector to be returned
	vector<fs::path> userPaths;
	char buffer[kStringSize];
	string pathStr;
	while(cin.getline(buffer, kStringSize))
	{
		pathStr = buffer;
		if(pathStr == "\0")
			return userPaths;
		expandSymbols(pathStr);
		fs::path tempPath(pathStr);

		//only add valid paths to vector
		if(fs::exists(tempPath))
			userPaths.push_back(tempPath);	

		else
			cout << "no such path: " << tempPath.string() << endl;

		
	}//end of while input		

	return userPaths;

} //end of getPathsFromUser function

//************************************************************************

